//
//  AppDelegate.m
//  HA7 logger mini
//  Copyright(C)2015 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2015.03.08 v0.0  First cording
//    2015.03.23 v0.5  Commit
//

#import "AppDelegate.h"

@interface AppDelegate ()
@property (nonatomic)   BOOL    displayOn;
@end


@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    NSLog(@"application didFinishLaunchingWithOptions: load prm");
    
    self.obd = [[ObdDataLogger alloc] init];
    self.obd.dist_s     = [[NSUserDefaults standardUserDefaults] doubleForKey:@"dist_s"];
    self.obd.fuse_s     = [[NSUserDefaults standardUserDefaults] doubleForKey:@"fuse_s"];
    self.obd.runtm      = (long)([[NSUserDefaults standardUserDefaults] doubleForKey:@"runtm"] * 3600.0);
    self.obd.movtm      = (long)([[NSUserDefaults standardUserDefaults] doubleForKey:@"movtm"] * 3600.0);
    self.obd.remainFuel = [[NSUserDefaults standardUserDefaults] doubleForKey:@"remainFuel"];

    // Sweep of temporary files
    NSString *tmpDir = NSTemporaryDirectory();
    NSDirectoryEnumerator *directoryEnumerator = [[NSFileManager defaultManager] enumeratorAtPath:tmpDir];
    for (NSString *file in directoryEnumerator) {
        NSString *path = [tmpDir stringByAppendingString:file];
        [[NSFileManager defaultManager] removeItemAtPath:path error:nil];
        NSLog(@"Sweep=%@",path);
    }
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    NSLog(@"applicationWillResignActive: save prm");
    
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    [defaults setDouble:self.obd.dist_s       forKey:@"dist_s"];
    [defaults setDouble:self.obd.fuse_s       forKey:@"fuse_s"];
    [defaults setDouble:self.obd.runtm/3600.0 forKey:@"runtm"];
    [defaults setDouble:self.obd.movtm/3600.0 forKey:@"movtm"];
    [defaults setDouble:self.obd.remainFuel   forKey:@"remainFuel"];
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    NSLog(@"applicationDidEnterBackground: sleep ok");
    [UIApplication sharedApplication].idleTimerDisabled = NO;
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    NSLog(@"applicationWillEnterForeground");
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    NSLog(@"applicationDidBecomeActive: disp=%d", self.displayOn);
    [UIApplication sharedApplication].idleTimerDisabled = self.displayOn;
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    NSLog(@"applicationWillTerminate");
}

- (void)setDisplayOn
{
    self.displayOn = YES;
    [UIApplication sharedApplication].idleTimerDisabled = self.displayOn;
}

- (void)unsetDisplayOn
{
    self.displayOn = NO;
    [UIApplication sharedApplication].idleTimerDisabled = self.displayOn;
}

@end
