//
//  DashboardViewController.m
//  HA7 logger mini
//  Copyright(C)2015-2016 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2015.03.10 v0.0  First cording
//    2015.03.23 v0.5  Commit
//    2016.11.27 v0.5b Bug fix (iOS 10~)
//

#include <math.h>
#import <CoreMotion/CoreMotion.h>
#import "DashboardViewController.h"
#import "AppDelegate.h"


@interface DashboardViewController () {
    AppDelegate             *appDeligate;
    ObdDataLogger           *obd;
    NSTimer                 *refreshTimer;
    int                      stopTm;
    enum {INIT,CONNECT,DISCONNECT} connectStat;

    UIImagePickerController *imagePicker;
    BOOL                     recording;
    BOOL                     saving;
    NSTimer                 *intervalTimer;
    NSTimer                 *refreshTimer2;
    NSString                *beforeVideoFile;
    CMMotionManager         *motionManager;
}
@end


@implementation DashboardViewController

//
//  Manage of standard view
//
- (void)viewDidLoad
{
    [super viewDidLoad];

    appDeligate  = [[UIApplication sharedApplication] delegate];
    obd          = appDeligate.obd;
    connectStat  = INIT;
    recording    = NO;
    saving       = NO;

    [self createImagePicker];
}


- (void)viewWillAppear:(BOOL)animated
{
    NSLog(@"viewWillAppear");
    [super viewWillAppear:animated];
    refreshTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(refreshView:) userInfo:nil repeats:YES];
}


- (void)viewWillDisappear:(BOOL)animated
{
    NSLog(@"viewWillDisappear");
    [super viewWillDisappear:animated];
    if (refreshTimer) {
        [refreshTimer invalidate];
        refreshTimer = nil;
    }
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


//
//  Instrument view
//
- (void)refreshView:(NSTimer *)timer
{
//    NSLog(@"refreshView");
    if (obd.valid) {
        if (obd.dist > 0.001) {   // 1[m/s]
            stopTm = 0;
        } else {
            stopTm++;
        }
        if (connectStat != CONNECT) {
            connectStat = CONNECT;
            [statMsg setText:@"online"];
            mainPanel.alpha = 1.0;
            subPanel.alpha  = 1.0;
            [self startRecord];
        }
    } else {
        if (connectStat != DISCONNECT) {
            connectStat = DISCONNECT;
            [statMsg setText:@"offline"];
            mainPanel.alpha = 0.5;
            subPanel.alpha  = 0.5;
            [self stopRecord];
        }
    }
    
    [vss        setText:[[NSString alloc] initWithFormat:@"%d", obd.vss]];
    [rpm        setText:[[NSString alloc] initWithFormat:@"%d", obd.rpm]];
    [rpmG       setIntValue:obd.rpm];
    
    [effNow     setText:[[NSString alloc] initWithFormat:@"%.1f", obd.eff]];
    [effDay     setText:[[NSString alloc] initWithFormat:@"%.1f", obd.eff_d]];
    [effAve     setText:[[NSString alloc] initWithFormat:@"%.1f", obd.eff_s]];
    [effNowG    setFloatValue:obd.eff];
    [effDayG    setFloatValue:obd.eff_d];
    [effAveG    setFloatValue:obd.eff_s];
    
    [stopTim    setText:[[NSString alloc] initWithFormat:@"%d:%02d", stopTm/60, stopTm%60]];
    [fuse_d     setText:[[NSString alloc] initWithFormat:@"%.2f", obd.fuse_d]];
    [dist_d     setText:[[NSString alloc] initWithFormat:@"%.2f", obd.dist_d]];
    [runtmDay   setText:[[NSString alloc] initWithFormat:@"%ld:%02ld", obd.runtm_d/60, obd.runtm_d%60]];
    [runtmDayG  setProgress:((double)obd.movtm_d/(double)obd.runtm_d)];
    
    [remainFuel setText:[[NSString alloc] initWithFormat:@"%.2f", obd.remainFuel]];
    [remainFuelG setProgress:(obd.remainFuel/FUEL_TANK)];
    [remainDist setText:[[NSString alloc] initWithFormat:@"%d",(int)(obd.remainFuel * obd.eff_s)]];
    [ect        setText:[[NSString alloc] initWithFormat:@"%d", obd.ect]];
    [ectG       setProgress:((obd.ect + 30.0)/130.0)];
    [vpwr       setText:[[NSString alloc] initWithFormat:@"%.1f",(float)obd.vpwr/1000.0]];
    [vpwrG      setProgress:((obd.vpwr - 11000)/4000.0)];
}


//
//  Camera view
//
- (void)createImagePicker
{
    if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera]) {
        imagePicker = [[UIImagePickerController alloc] init];
        imagePicker.delegate             = self;
        imagePicker.sourceType           = UIImagePickerControllerSourceTypeCamera;
        imagePicker.mediaTypes           = [NSArray arrayWithObject:@"public.movie"];
        imagePicker.cameraCaptureMode    = UIImagePickerControllerCameraCaptureModeVideo;
        imagePicker.allowsEditing        = NO;
        imagePicker.showsCameraControls  = NO;
        imagePicker.cameraViewTransform  = CGAffineTransformIdentity;
        imagePicker.videoQuality         = UIImagePickerControllerQualityTypeIFrame1280x720;
//      imagePicker.videoMaximumDuration = 60.0;    // UPDT 2016.11.27
        imagePicker.videoMaximumDuration = 600.0;   // UPDT 2016.11.27
        if ([UIImagePickerController isCameraDeviceAvailable: UIImagePickerControllerCameraDeviceRear]) {
            imagePicker.cameraDevice     = UIImagePickerControllerCameraDeviceRear;
        } else {
            imagePicker.cameraDevice     = UIImagePickerControllerCameraDeviceFront;
        }
        if ([UIImagePickerController isFlashAvailableForCameraDevice:imagePicker.cameraDevice]) {
            imagePicker.cameraFlashMode  = UIImagePickerControllerCameraFlashModeOff;
        }
        imagePicker.cameraOverlayView = cameraOverlayView;
        [recordMsg setText:@"Ready"];
    } else {
        [recordMsg setText:@"No camera"];
        recordSw.alpha  = 0.0;
        recordSwL.alpha = 0.0;
        saveSw.alpha    = 0.0;
        saveSwL.alpha   = 0.0;
    }
}


- (void)startRecord
{
    NSLog(@"startRecord");
    if (imagePicker) {
        if (!recording) {
            CGRect r = [cameraOverlayView frame];         // reset of view position
            [imagePicker.view setFrame:r];
            UIInterfaceOrientation o1 = [[UIApplication sharedApplication] statusBarOrientation];
            // Takeover the front view (image picker view)
            [self presentViewController:imagePicker animated:YES completion:nil];
            UIInterfaceOrientation o2 = [[UIApplication sharedApplication] statusBarOrientation];
            if (o1 != o2) {     // changing a display direction of overlay view by force
                if (o1 == UIInterfaceOrientationLandscapeLeft) {
                    NSLog(@"Force landscape L mode");
                    cameraOverlayView.transform = CGAffineTransformMakeRotation(M_PI / -2.0);
                } else if (o1 == UIInterfaceOrientationLandscapeRight) {
                    cameraOverlayView.transform = CGAffineTransformMakeRotation(M_PI / 2.0);
                    NSLog(@"Force landscape R mode");
                }
            }
            refreshTimer2 = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(refreshView:) userInfo:nil repeats:YES];
            intervalTimer = [NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(waitRecording:) userInfo:nil repeats:YES];
            recording = YES;
            
            motionManager = [[CMMotionManager alloc] init];
            if ([motionManager isAccelerometerAvailable]) {
                motionManager.accelerometerUpdateInterval = 0.1;    // [sec]
                [motionManager startAccelerometerUpdatesToQueue:[NSOperationQueue mainQueue] withHandler:^(CMAccelerometerData *accelerometerData, NSError *error) {
                    if (accelerometerData.acceleration.z > SAVE_START_G ||
                        accelerometerData.acceleration.z < -SAVE_START_G) {
                        [self startSave];
                    }
                }];
            } else {
                motionManager = nil;
            }
        }
        [recordSw setOn:YES];
    }
}


- (void)waitRecording:(NSTimer *)timer
{
    NSLog(@"waitRecording");
    if ([imagePicker startVideoCapture]) {
        [intervalTimer invalidate];
        intervalTimer = [NSTimer scheduledTimerWithTimeInterval:RECORD_PERIOD target:self selector:@selector(changeRecord:) userInfo:nil repeats:YES];
        [appDeligate setDisplayOn];
        [recordMsg setText:@"REC"];
    } else {
        [recordMsg setText:@"Wait"];
    }
}


- (void)changeRecord:(NSTimer *)timer
{
    NSLog(@"changeRecord");
    if (recording) {
        [recordMsg setText:@"Busy"];
        [imagePicker stopVideoCapture];
//      if ([imagePicker startVideoCapture]) {      // DEL 2016.11.27
//          NSLog(@"Error recovery is success");    // DEL 2016.11.27
//      }                                           // DEL 2016.11.27
    }
}


- (void)stopRecord
{
    NSLog(@"stopRecord");
    if (imagePicker) {
        if (recording) {
            [recordMsg setText:@"STOP"];
            
            if (motionManager) {
                [motionManager stopAccelerometerUpdates];
            }

            [intervalTimer invalidate];
            intervalTimer = nil;
            [refreshTimer2 invalidate];
            refreshTimer2 = nil;
            NSLog(@"Timer stop");
            [imagePicker stopVideoCapture];
        }
        [recordSw setOn:NO];
        [saveSw setOn:NO];
        [appDeligate unsetDisplayOn];
    }
}


- (void)startSave
{
    if (imagePicker) {
        if (!saving) {
            if (beforeVideoFile) {
                UISaveVideoAtPathToSavedPhotosAlbum(beforeVideoFile, self, @selector(video:didFinishSavingWithError:contextInfo:), NULL);
                beforeVideoFile = nil;
            }
            saving = YES;
            [self startRecord];
        }
        [saveSw setOn:YES];
    }
}


- (void)stopSave
{
    if (imagePicker) {
        if (saving) {
            [imagePicker stopVideoCapture];
        }
        [saveSw setOn:NO];
    }
}


- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    NSURL    *videoURL    = [info valueForKey:UIImagePickerControllerMediaURL];
    NSString *pathToVideo = [videoURL path];

    NSLog(@"Create=%@", pathToVideo);
    if (beforeVideoFile) {
        NSLog(@"Remove=%@", beforeVideoFile);
        [[NSFileManager defaultManager] removeItemAtPath:beforeVideoFile error:nil];
//      [[NSFileManager defaultManager] removeItemAtPath:[beforeVideoFile stringByDeletingLastPathComponent] error:nil];        // DEL 2016.11.27
    }
    
    if (saving) {           // Permanent record
        UISaveVideoAtPathToSavedPhotosAlbum(pathToVideo, self, @selector(video:didFinishSavingWithError:contextInfo:), NULL);
    }

    if (!recordSw.on) {
        cameraOverlayView.transform = CGAffineTransformIdentity;
        [self dismissViewControllerAnimated:YES completion:nil];
        [recordMsg setText:@"Ready"];
        recording = NO;
    }

    if (recording) {
        if ([imagePicker startVideoCapture]) {
            NSLog(@"RESTART OK");
            [recordMsg setText:@"REC"];
        } else {
            NSLog(@"RESTART ABORT");
            [recordMsg setText:@"ERR"];
        }
    }

    beforeVideoFile = pathToVideo;
}


- (void)video:(NSString *)videoPath didFinishSavingWithError:(NSError *)error contextInfo:(void *)contextInfo {
    NSLog(@"Movie saved & remove: %@", videoPath);
    [[NSFileManager defaultManager] removeItemAtPath:videoPath error:nil];
//  [[NSFileManager defaultManager] removeItemAtPath:[videoPath stringByDeletingLastPathComponent] error:nil];      // DEL 2016.11.27
    if (!saveSw.on) {
        saving = NO;
    }
}


- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    NSLog(@"imagePickerControllerDidCancel");
    [self dismissViewControllerAnimated:YES completion:nil];
}


- (IBAction)slidedRecordSw:(id)sender{
    NSLog(@"slidedRecordSw");
    if (recordSw.on) {
        [self startRecord];
    } else {
        [self stopRecord];
    }
}


- (IBAction)slidedSaveSw:(id)sender{
    NSLog(@"slidedSaveSw");
    if (saveSw.on) {
        [self startSave];
    } else {
        [self stopSave];
    }
}


- (IBAction)exit1:(id)sender
{
    if (refreshTimer) {
        [refreshTimer invalidate];
        refreshTimer = nil;
    }
    if (saving) {
        [self stopSave];
    } else if (recording) {
        [self stopRecord];
    } else {
        if (beforeVideoFile) {
            NSLog(@"Remove=%@", beforeVideoFile);
            [[NSFileManager defaultManager] removeItemAtPath:beforeVideoFile error:nil];
//          [[NSFileManager defaultManager] removeItemAtPath:[beforeVideoFile stringByDeletingLastPathComponent] error:nil];        // DEL 2016.11.27
            beforeVideoFile = nil;
        }
        [self dismissViewControllerAnimated:YES completion:nil];
        UINavigationController *n = [self navigationController];
        [n popViewControllerAnimated:YES];
    }
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
