//
//  HistoryCollectionViewController.m
//  HA7 logger mini
//  Copyright(C)2015 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2015.03.08 v0.0  First cording
//    2015.03.23 v0.5  Commit
//    2015.03.31 v0.5a Bug fix
//

#import "HistoryCollectionViewController.h"
#import "CalendarViewCell.h"
#import "RouteMapViewController.h"

@interface HistoryCollectionViewController ()

@property (strong, nonatomic)   NSDate           *calendarTopDate;
@property (strong, nonatomic)   UICollectionView *cv;
@property (nonatomic)           double            beforeMonth;

@end

@implementation HistoryCollectionViewController

// Internal function
- (void)changeCalendarTopDate:(NSDate *)calendarTopDate
{
    self.calendarTopDate = calendarTopDate;
    NSDate *ym = [[NSDate alloc] initWithTimeInterval:24*60*60*7 sinceDate:self.calendarTopDate];
    NSDateComponents *ymComps = [[NSCalendar currentCalendar] components:NSCalendarUnitYear | NSCalendarUnitMonth fromDate:ym];
    [self.yyyymm setText:[[NSString alloc] initWithFormat:@"%d/%d", (int)ymComps.year, (int)ymComps.month]];
    [self setTitle:[[NSString alloc] initWithFormat:@"%d/%d", (int)ymComps.year, (int)ymComps.month]];
}


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    
    NSDate *now = [[NSDate alloc] init];
    NSDateComponents *nowComps = [[NSCalendar currentCalendar] components:NSCalendarUnitYear|NSCalendarUnitMonth|NSCalendarUnitDay|NSCalendarUnitWeekday fromDate:now];
    NSDate *calTop = [[NSDate alloc] initWithTimeInterval:-24*60*60*((nowComps.day-nowComps.weekday+6)/ 7 * 7 +nowComps.weekday - 1) sinceDate:now];
    [self changeCalendarTopDate:calTop];
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}


- (void)viewWillAppear:(BOOL)animated
{
    self.beforeMonth = self.monthUpDown.value;
    [super viewWillAppear:animated];
}


- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}


- (NSInteger)collectionView:(UICollectionView *)view numberOfItemsInSection:(NSInteger)section
{
    self.cv = view;
    return 7*5;
}


- (UICollectionViewCell *)collectionView:(UICollectionView *)cv cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    
    CalendarViewCell *cell = [cv dequeueReusableCellWithReuseIdentifier:@"Cell1" forIndexPath:indexPath];
    NSDate *d = [[NSDate alloc] initWithTimeInterval:24*60*60*indexPath.row sinceDate:self.calendarTopDate];
    NSDateComponents *dc = [[NSCalendar currentCalendar] components:NSCalendarUnitYear|NSCalendarUnitMonth|NSCalendarUnitDay fromDate:d];
    [cell.monthOfdate setText:[NSString stringWithFormat:@"%d", (int)dc.day]];
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path1 = [[NSString alloc] initWithFormat:@"%@/%d-%02d-%02d.csv", [paths objectAtIndex:0], (int)dc.year, (int)dc.month, (int)dc.day];
    const char *fname = [path1 UTF8String];
    FILE *fp = fopen(fname,"r");
    if (fp != NULL) {
//      char    buff[300];              // DEL 2015.03.31
        char    buff[400];              // ADD 2015.03.31
        char    *p;
        char    *pp;
        int     i;
        double  dist_b;
        double  fuse_b;
        double  dist_e;
        double  fuse_e;
        double  dist;
        double  fuse;
        float   eff;
        
        dist_e = 0.0;
        dist_b = 0.0;
        fuse_e = 0.0;
        fuse_b = 0.0;
        fgets(buff, sizeof(buff), fp);
        fgets(buff, sizeof(buff), fp);
        for(p = strtok_r(buff, ",", &pp), i = 0; p != NULL; p = strtok_r(NULL, ",", &pp), i++) {
            if (i == 16) {
                dist_b = atof(p);
            } else if (i == 17) {
                fuse_b = atof(p);
            }
        }
        fseek(fp, -400L, SEEK_END);
        fgets(buff, sizeof(buff), fp);
        fgets(buff, sizeof(buff), fp);
        for(p = strtok_r(buff, ",", &pp), i = 0; p != NULL; p = strtok_r(NULL, ",", &pp), i++) {
            if (i == 16) {
                dist_e = atof(p);
            } else if (i == 17) {
                fuse_e = atof(p);
            }
        }
        fclose(fp);
        dist = dist_e - dist_b;
        fuse = fuse_e - fuse_b;
        if ((dist_e >= 0.0) && (fuse > 0.0)) {
            eff = dist / fuse;
        } else {
            eff = 0.0;
        }
        switch(self.item.selectedSegmentIndex){
            case 0:
                [cell.value setText:[NSString stringWithFormat:@"%.1f",eff]];
                break;
            case 1:
                [cell.value setText:[NSString stringWithFormat:@"%.1f",fuse]];
                break;
            case 2:
                [cell.value setText:[NSString stringWithFormat:@"%.1f",dist]];
                break;
            default:
                break;
        }
    } else {
        [cell.value setText:nil];
    }
    
    if ((indexPath.row < 7) && (dc.day > 7)) {
        cell.monthOfdate.alpha = 0.3;
        cell.value.alpha       = 0.3;
    } else if ((indexPath.row > 28) && (dc.day < 15)) {
        cell.monthOfdate.alpha = 0.3;
        cell.value.alpha       = 0.3;
    } else {
        cell.monthOfdate.alpha = 1.0;
        cell.value.alpha       = 1.0;
    }
    return cell;
}


- (IBAction)nextMonth:(id)sender
{
    NSDate *ym = [[NSDate alloc] initWithTimeInterval:24*60*60*42 sinceDate:self.calendarTopDate];
    NSDateComponents *nowComps = [[NSCalendar currentCalendar] components:NSCalendarUnitYear|NSCalendarUnitMonth|NSCalendarUnitDay|NSCalendarUnitWeekday fromDate:ym];
    NSDate *calTop = [[NSDate alloc] initWithTimeInterval:-24*60*60*((nowComps.day-nowComps.weekday+6)/ 7 * 7 +nowComps.weekday - 1) sinceDate:ym];
    NSDate *today = [[NSDate alloc] init];
    if ([calTop compare:today] == NSOrderedAscending) {
        [self changeCalendarTopDate:calTop];
        [self.cv reloadData];
    }
}


- (IBAction)prevMonth:(id)sender
{
    NSDate *ym = [[NSDate alloc] initWithTimeInterval:-24*60*60*7 sinceDate:self.calendarTopDate];
    NSDateComponents *nowComps = [[NSCalendar currentCalendar] components:NSCalendarUnitYear|NSCalendarUnitMonth|NSCalendarUnitDay|NSCalendarUnitWeekday fromDate:ym];
    NSDate *calTop = [[NSDate alloc] initWithTimeInterval:-24*60*60*((nowComps.day-nowComps.weekday+6)/ 7 * 7 +nowComps.weekday - 1) sinceDate:ym];
    [self changeCalendarTopDate:calTop];
    [self.cv reloadData];
}


- (IBAction)changeMonth:(id)sender
{
    if (self.beforeMonth < self.monthUpDown.value) {
        [self nextMonth:nil];
    } else {
        [self prevMonth:nil];
    }
    self.beforeMonth = self.monthUpDown.value;
}


- (IBAction)changeItem:(id)sender
{
    self.item = sender;
    [self.cv reloadData];
}


- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"showDetail"]){
        NSIndexPath *selectedIndexPath = [[self.cv indexPathsForSelectedItems] objectAtIndex:0];
        NSDate *d = [[NSDate alloc] initWithTimeInterval:24*60*60*selectedIndexPath.row sinceDate:self.calendarTopDate];
        
        RouteMapViewController *c = [segue destinationViewController];
        NSLog(@"prepareForSegue: %@", c.showDate);
        c.showDate = d;
    }
}

- (UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath
{
    UICollectionReusableView *r = nil;
    
    if (kind == UICollectionElementKindSectionHeader) {
        r = [collectionView dequeueReusableSupplementaryViewOfKind:kind withReuseIdentifier:@"Head1" forIndexPath:indexPath];
    } else if (kind == UICollectionElementKindSectionFooter) {
        r = [collectionView dequeueReusableSupplementaryViewOfKind:kind withReuseIdentifier:@"Foot1" forIndexPath:indexPath];
    }
    return(r);
}


@end
